SystemOrganization addCategory: #'EjerciciosPoo-Measure'!

!Interval methodsFor: '*EjerciciosPoo-Measure' stamp: 'PabloRago 5/26/2013 16:49'!
do: aBlock
	"Evaluate aBlock for each value of the interval.
	Implementation note: instead of repeatedly incrementing the value
		aValue := aValue + step.
	until stop is reached,
	We prefer to recompute value from start
		aValue := start + (index * step).
	This is better for floating points accuracy, while not degrading Integer and Fraction speed too much.
	Moreover, this is consistent with methods #at: and #size"

	| aValue index size |
	index := 0.
	size := self size.
	[index < size]
		whileTrue: [aValue := start + (step * index).
			index := index + 1.
			aBlock value: aValue]! !

!Interval methodsFor: '*EjerciciosPoo-Measure' stamp: 'PabloRago 5/26/2013 17:11'!
last 
	"Refer to the comment in SequenceableCollection|last."

	^stop! !

!Interval methodsFor: '*EjerciciosPoo-Measure' stamp: 'PabloRago 5/26/2013 16:49'!
size
	"Answer how many elements the receiver contains."

	step negative
		ifTrue: [start < stop
				ifTrue: [^ 0]
				ifFalse: [^ stop - start // step + 1]]
		ifFalse: [stop < start
				ifTrue: [^ 0]
				ifFalse: [^ stop - start // step + 1]]! !

!String methodsFor: '*EjerciciosPoo-Measure' stamp: 'MartinCarreiro 5/24/2013 09:25'!
asPluralBasedOn: aNumberOrCollection
	"Append an 's' to this string based on whether aNumberOrCollection is 1 or of size 1."

	^ ((aNumberOrCollection = 1 or:
		[aNumberOrCollection isCollection and: [aNumberOrCollection size = 1]]) or: self last = $s)
			ifTrue: [self]
			ifFalse: [self, 's']
! !

Magnitude subclass: #Measure
	instanceVariableNames: 'amount unit'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-Measure'!

!Measure class methodsFor: 'as yet unclassified' stamp: 'PabloRago 5/8/2013 19:35'!
newMeasure: aValue withUnit: aUnit 

	|aMeasure|

	 aMeasure := self new.
	^aMeasure initializeMeasure: aValue withUnit: aUnit. ! !

!Measure methodsFor: 'arithmetic' stamp: 'MartinCarreiro 5/13/2013 20:13'!
* anEscalar
	
	^ Measure newMeasure: ((self amount) * anEscalar ) withUnit: self unit.! !

!Measure methodsFor: 'arithmetic' stamp: 'PabloRago 5/26/2013 22:34'!
+ anotherMeasure
	
	self signalIfDifferentDomain: anotherMeasure unit.
	^ Measure newMeasure: (self amount + ((anotherMeasure convertTo: unit)amount) ) withUnit: unit .! !

!Measure methodsFor: 'arithmetic' stamp: 'MartinCarreiro 5/13/2013 20:14'!
- anotherMeasure
	
	^ self + anotherMeasure negated.! !

!Measure methodsFor: 'arithmetic' stamp: 'MartinCarreiro 5/13/2013 20:14'!
/ anEscalar
	
	^self * (1/anEscalar).! !

!Measure methodsFor: 'arithmetic' stamp: 'PabloRago 5/15/2013 19:47'!
// anotherMeasure
	
	^self amountNormalized // anotherMeasure amountNormalized .! !

!Measure methodsFor: 'comparing' stamp: 'PabloRago 5/26/2013 17:29'!
< anotherMeasure
	
	^(self amountNormalized) < (anotherMeasure amountNormalized).! !

!Measure methodsFor: 'comparing' stamp: 'PabloRago 5/26/2013 21:54'!
= anotherMeasure

	^ (((anotherMeasure isKindOf: Measure)and: [ unit domain = anotherMeasure unit domain] ) and: [(self amountNormalized) = (anotherMeasure amountNormalized). ])or:
	[(self amountNormalized = 0 and: [anotherMeasure amountNormalized = 0]) and: [(self unit = self unit normal) and: [anotherMeasure unit = anotherMeasure unit normal.].].] . ! !

!Measure methodsFor: 'arithmetic' stamp: 'PabloRago 5/15/2013 19:47'!
\\ anotherMeasure
	
	^self amountNormalized \\ anotherMeasure amountNormalized .! !

!Measure methodsFor: 'getters' stamp: 'PabloRago 5/26/2013 15:26'!
amount 

	^amount asFraction.! !

!Measure methodsFor: 'getters' stamp: 'PabloRago 5/20/2013 21:29'!
amountNormalized
	"^(amount / (10**unit factor))."
	"^(amount *  unit factor)."
	^ unit normalize: amount! !

!Measure methodsFor: 'converting' stamp: 'MartinCarreiro 5/24/2013 14:47'!
convertTo: aUnit
	
	|newAmount|
	
	
	
	self signalIfDifferentDomain: aUnit.
	newAmount := self unit convertAmount: amount to: aUnit.
	^Measure newMeasure: newAmount withUnit: aUnit.! !

!Measure methodsFor: 'initialize-release' stamp: 'MartinCarreiro 5/13/2013 20:14'!
copy
	
	^ Measure newMeasure: self amount withUnit: (self unit)
! !

!Measure methodsFor: 'getters' stamp: 'MartinCarreiro 5/13/2013 19:37'!
hash

	"Multiplicamos por 31 porque es primo y porque somos 3 en el grupo pero parecemos 1 :) "
	^self amountNormalized *31.! !

!Measure methodsFor: 'initialize-release' stamp: 'PabloRago 5/8/2013 20:17'!
initializeMeasure: anAmount withUnit: aUnit
	amount := anAmount.
	unit := aUnit.! !

!Measure methodsFor: 'arithmetic' stamp: 'MartinCarreiro 5/13/2013 20:15'!
negated
	
	^ Measure newMeasure: (self amount) * (-1) withUnit: self unit.! !

!Measure methodsFor: 'arithmetic' stamp: 'PabloRago 5/15/2013 21:47'!
negative

	^ self amount negative.! !

!Measure methodsFor: 'getters' stamp: 'MartinCarreiro 5/13/2013 21:08'!
oneMeasureWithMyUnit

	^ Measure newMeasure: 1 withUnit: self unit.! !

!Measure methodsFor: 'getters' stamp: 'PabloRago 5/8/2013 20:33'!
printString
	^ amount asString , ' ' , unit printString asPluralBasedOn: amount .! !

!Measure methodsFor: 'arithmetic' stamp: 'MartinCarreiro 5/24/2013 14:47'!
signalIfDifferentDomain: aUnit
	[
	self assert: (unit domain = aUnit domain ).
	]
	on: AssertionFailure
	do: [ CanNotConvertMeasurementException signal ].! !

!Measure methodsFor: 'intervals' stamp: 'PabloRago 5/15/2013 21:51'!
to: aMeasure

	^Interval from: self  to: aMeasure by: self oneMeasureWithMyUnit! !

!Measure methodsFor: 'intervals' stamp: 'PabloRago 5/15/2013 19:43'!
to: aMeasure by: aMeasureStep

	^Interval from: self  to: aMeasure by: aMeasureStep ! !

!Measure methodsFor: 'getters' stamp: 'PabloRago 5/8/2013 19:49'!
unit
	^unit.! !

TestCase subclass: #MeasureTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-Measure'!

!MeasureTest methodsFor: 'errors' stamp: 'MartinCarreiro 5/24/2013 14:50'!
canNotConvertMeasurementException
	
"	self error: 'Este metodo debe devolver el objeto que representa que no se pudo
		convertir una medida a la unidad pedida'"
		
	^CanNotConvertMeasurementException.! !

!MeasureTest methodsFor: 'temperature units' stamp: 'PabloRago 5/20/2013 20:39'!
celsius
	
	^Unit celsius.! !

!MeasureTest methodsFor: 'distance units' stamp: 'PabloRago 5/20/2013 20:27'!
centimeter
	
	^ Unit centimeter.! !

!MeasureTest methodsFor: 'time units' stamp: 'PabloRago 5/20/2013 20:42'!
day
	
	^Unit day.! !

!MeasureTest methodsFor: 'distance units' stamp: 'PabloRago 5/20/2013 20:27'!
decameter
	
	^ Unit decameter.! !

!MeasureTest methodsFor: 'distance units' stamp: 'PabloRago 5/20/2013 20:27'!
decimeter
	
	^ Unit decimeter.! !

!MeasureTest methodsFor: 'distance measures' stamp: 'HernanWilkinson 4/25/2011 17:27'!
distanceExamples 

	^self objectsInCategoryOf: thisContext method selector.
	
! !

!MeasureTest methodsFor: 'distance units' stamp: 'HernanWilkinson 4/25/2011 17:27'!
distanceUnits 

	^self objectsInCategoryOf: thisContext method selector.
	
! !

!MeasureTest methodsFor: 'compound measures' stamp: 'HernanWilkinson 4/25/2011 17:35'!
eightCubicMeters

	^self error: 'este metodo debe devolver la medida 8 metros cubicos'! !

!MeasureTest methodsFor: 'temperature units' stamp: 'PabloRago 5/20/2013 20:39'!
fahrenheit
	
	^Unit fahrenheit.! !

!MeasureTest methodsFor: 'compound measures' stamp: 'HernanWilkinson 4/25/2011 17:40'!
finalSpeedStartingAt: initialSpeed accelerating: anAcceleration during: aTimePeriod

	^self error: 'Implementar la formula de velocidad final: vf = vi + a * t'! !

!MeasureTest methodsFor: 'compound measures' stamp: 'HernanWilkinson 4/25/2011 17:35'!
fourSquaredMeters

	^self error: 'este metodo debe devolver la medida 4 metros cuadrados'! !

!MeasureTest methodsFor: 'distance units' stamp: 'PabloRago 5/20/2013 20:28'!
hectometer
	
	^ Unit hectometer.! !

!MeasureTest methodsFor: 'time units' stamp: 'PabloRago 5/20/2013 20:42'!
hour
	
	^Unit hour.! !

!MeasureTest methodsFor: 'temperature units' stamp: 'PabloRago 5/20/2013 20:39'!
kelvin
	
	^Unit kelvin.! !

!MeasureTest methodsFor: 'distance units' stamp: 'PabloRago 5/20/2013 20:28'!
kilometer
	
	^ Unit kilometer.! !

!MeasureTest methodsFor: 'objects in category' stamp: 'HernanWilkinson 4/25/2011 17:30'!
measureExamples 

	^ OrderedCollection new
		addAll: self distanceExamples;
		addAll: self temperatureExamples;
		addAll: self timeExamples;
		yourself ! !

!MeasureTest methodsFor: 'distance units' stamp: 'PabloRago 5/20/2013 20:28'!
meter

	^ Unit meter.! !

!MeasureTest methodsFor: 'distance units' stamp: 'PabloRago 5/20/2013 20:28'!
millimeter
	
	^ Unit millimeter.! !

!MeasureTest methodsFor: 'distance measures' stamp: 'PabloRago 5/20/2013 20:33'!
minusOneMeter 
	
	^ Measure newMeasure: -1 withUnit: self meter ! !

!MeasureTest methodsFor: 'time units' stamp: 'PabloRago 5/20/2013 20:42'!
minute
	
	^Unit minute.! !

!MeasureTest methodsFor: 'objects in category' stamp: 'HernanWilkinson 4/25/2011 17:27'!
objectsInCategoryOf: aSelector

	| category messageNames |
	
	category := self class organization categoryOfElement: aSelector.
	messageNames := (self class organization listAtCategoryNamed:  category) reject: [ :aMessageName | aMessageName = aSelector ].
	
	 ^ messageNames collect: [ :aMessageName | self perform: aMessageName ]
	

	
! !

!MeasureTest methodsFor: 'temperature measures' stamp: 'PabloRago 5/20/2013 20:43'!
oneCelsius
	
	^ Measure newMeasure: 1 withUnit: self celsius ! !

!MeasureTest methodsFor: 'distance measures' stamp: 'PabloRago 5/20/2013 20:31'!
oneCentimeter
	
	^ Measure newMeasure: 1 withUnit: self centimeter ! !

!MeasureTest methodsFor: 'time measures' stamp: 'PabloRago 5/20/2013 20:46'!
oneDay
	
	^ Measure newMeasure: 1 withUnit: self day ! !

!MeasureTest methodsFor: 'time measures' stamp: 'PabloRago 5/20/2013 20:46'!
oneHour
	
	^ Measure newMeasure: 1 withUnit: self hour ! !

!MeasureTest methodsFor: 'distance measures' stamp: 'PabloRago 5/20/2013 20:33'!
oneHundredCentimeters
	
	^ Measure newMeasure: 100 withUnit: self centimeter ! !

!MeasureTest methodsFor: 'compound measures' stamp: 'efo 5/2/2011 20:56'!
oneHundredTwentyOneMetersPerSecond

	^self error: 'este metodo debe devolver la medida 121 metro/segundo'! !

!MeasureTest methodsFor: 'distance measures' stamp: 'PabloRago 5/20/2013 20:33'!
oneMeter
	
	^ Measure newMeasure: 1 withUnit: self meter ! !

!MeasureTest methodsFor: 'compound measures' stamp: 'HernanWilkinson 4/25/2011 17:36'!
oneMeterPerSecond

	^self error: 'este metodo debe devolver la medida 1 metro/segundo'! !

!MeasureTest methodsFor: 'time measures' stamp: 'PabloRago 5/20/2013 20:47'!
oneMinute
	
	^ Measure newMeasure: 1 withUnit: self minute ! !

!MeasureTest methodsFor: 'time measures' stamp: 'PabloRago 5/20/2013 20:46'!
oneSecond

	^ Measure newMeasure: 1 withUnit: self second ! !

!MeasureTest methodsFor: 'compound measures' stamp: 'HernanWilkinson 4/25/2011 17:37'!
oneSquaredMeter

	^self error: 'este metodo debe devolver la medida 1 metro cuadrado'! !

!MeasureTest methodsFor: 'compound measures' stamp: 'HernanWilkinson 4/25/2011 17:36'!
oneSquaredSecond

	^self error: 'este metodo debe devolver la medida 1 segundo cuadrado'! !

!MeasureTest methodsFor: 'time units' stamp: 'PabloRago 5/20/2013 20:42'!
second
	
	^Unit second.! !

!MeasureTest methodsFor: 'time measures' stamp: 'PabloRago 5/20/2013 20:47'!
sixtyMinutes
	
	^ Measure newMeasure: 60 withUnit: self minute ! !

!MeasureTest methodsFor: 'time measures' stamp: 'PabloRago 5/20/2013 20:47'!
sixtyOneSeconds
	
	^ Measure newMeasure: 61 withUnit: self second ! !

!MeasureTest methodsFor: 'time measures' stamp: 'PabloRago 5/20/2013 20:47'!
sixtySeconds
	
	^ Measure newMeasure: 60 withUnit: self second ! !

!MeasureTest methodsFor: 'temperature measures' stamp: 'HernanWilkinson 4/25/2011 17:28'!
temperatureExamples

	^self objectsInCategoryOf: thisContext method selector.
	
! !

!MeasureTest methodsFor: 'temperature units' stamp: 'HernanWilkinson 4/25/2011 17:28'!
temperatureUnits

	^self objectsInCategoryOf: thisContext method selector.
	
! !

!MeasureTest methodsFor: '1 - Test equality' stamp: 'HernanWilkinson 4/25/2011 16:05'!
test01ObjectsRepresentingSameMeasureShouldBeEqual

	self assert: self oneMeter = self oneMeter.
	self assert: self oneSecond = self oneSecond.
	self assert: self oneCelsius = self oneCelsius.
	

! !

!MeasureTest methodsFor: '1 - Test equality' stamp: 'HernanWilkinson 4/25/2011 16:06'!
test02DifferentMeasuresShouldNotBeEqual

	self deny: self oneMeter = self twoMeters.
	self deny: self oneSecond = self sixtySeconds.
	self deny: self oneCelsius = self zeroCelsius.
! !

!MeasureTest methodsFor: '1 - Test equality' stamp: 'HernanWilkinson 4/25/2011 16:07'!
test03SameMeasureExpressWithDifferentUnitsShouldBeEqual

	self assert: self oneMeter = self oneHundredCentimeters.
	self assert: self twoMeters = self twoHundredCentimeters.
	self assert: self oneMinute = self sixtySeconds.
	self assert: self zeroCelsius = self thirtyTwoFahrenheits.
! !

!MeasureTest methodsFor: '1 - Test equality' stamp: 'HernanWilkinson 4/25/2011 16:08'!
test03aMeasuresOfDifferentDomainShouldNotBeEqual

	self deny: self oneMeter = self oneSecond.
	self deny: self oneMeter = self oneCelsius.
	self deny: self oneCelsius = self oneSecond
! !

!MeasureTest methodsFor: '1 - Test equality' stamp: 'HernanWilkinson 4/25/2011 16:09'!
test04EqualsShouldBeCommutative

	self assert: (self oneMeter = self oneHundredCentimeters and: [ self oneHundredCentimeters = self oneMeter ]).
	self assert: (self twoMeters = self twoHundredCentimeters and:  [ self twoHundredCentimeters = self twoMeters ]).
	self assert: (self oneMinute = self sixtySeconds and: [self sixtySeconds = self oneMinute]).
	self assert: (self zeroCelsius = self thirtyTwoFahrenheits and: [ self thirtyTwoFahrenheits = self zeroCelsius]).
! !

!MeasureTest methodsFor: '2 - Converting/accessing/printing' stamp: 'HernanWilkinson 4/25/2011 16:11'!
test05UnitStringRepresentationShouldBeHumanReadable

	self assert: self meter printString = 'meter'.
	self assert: self centimeter printString = 'centimeter'.
	self assert: self decimeter printString = 'decimeter'.
	self assert: self millimeter printString = 'milimeter'.
	self assert: self decameter printString = 'decameter'.
	self assert: self hectometer printString = 'hectometer'.
	self assert: self kilometer printString = 'kilometer'.	
	
	self assert: self celsius printString = 'celsius'.	
	self assert: self fahrenheit printString = 'fahrenheit'.	

	self assert: self day printString = 'day'.	
	self assert: self hour printString = 'hour'.	
	self assert: self minute printString = 'minute'.	
	self assert: self second printString = 'second'.	

! !

!MeasureTest methodsFor: '2 - Converting/accessing/printing' stamp: 'MartinCarreiro 5/24/2013 09:15'!
test06MeasureStringRepresentationShouldBeHumanReadable

	self assert: self oneMeter printString = '1 meter'.
	self assert: self twoMeters printString = '2 meters'.
	self assert: self oneHundredCentimeters printString = '100 centimeters'.
	
	self assert: self oneMinute printString = '1 minute'.
	self assert: self sixtySeconds printString = '60 seconds'.
	
	self assert: self zeroCelsius printString = '0 celsius'.
	self assert: self thirtyTwoFahrenheits printString = '32 fahrenheits'.
	
	
! !

!MeasureTest methodsFor: '2 - Converting/accessing/printing' stamp: 'HernanWilkinson 4/25/2011 16:14'!
test07aMeasureAmountShouldBeTheExpectedAmount

	self assert: self oneMeter amount = 1.
	self assert: self twoMeters amount = 2.
	self assert: self oneHundredCentimeters amount = 100.
	self assert: self twoHundredCentimeters amount = 200.

	self assert: self oneMinute amount = 1.
	self assert: self sixtySeconds amount = 60.
	
	self assert: self zeroCelsius amount = 0.
	self assert: self thirtyTwoFahrenheits amount = 32.
! !

!MeasureTest methodsFor: '2 - Converting/accessing/printing' stamp: 'HernanWilkinson 4/25/2011 16:15'!
test07bMeasureUnitShouldBeTheExpectedUnit

	self assert: self oneMeter unit = self meter.
	self assert: self twoMeters unit = self meter.
	self assert: self oneHundredCentimeters unit = self centimeter.
	self assert: self twoHundredCentimeters unit = self centimeter.
	
	self assert: self oneMinute unit = self minute.
	self assert: self sixtySeconds unit = self second.
	
	self assert: self zeroCelsius unit = self celsius.
	self assert: self thirtyTwoFahrenheits unit = self fahrenheit
! !

!MeasureTest methodsFor: '2 - Converting/accessing/printing' stamp: 'HernanWilkinson 4/25/2011 16:15'!
test07cDifferentMeasureUnitsShouldBeDifferent

	| unitsGroupedByItself |
	
	unitsGroupedByItself := self units groupedBy: [ :aUnit | aUnit ].
	self assert: (unitsGroupedByItself allSatisfy: [:aCollectionOfUnit | aCollectionOfUnit size = 1] )
! !

!MeasureTest methodsFor: '2 - Converting/accessing/printing' stamp: 'MartinCarreiro 5/24/2013 10:11'!
test07dDistancesCanBeConvertedToDifferentUnits

	"A partir de los test de igualdad, se deben haber dado cuenta que debemos poder
	convertir distancias a distintas unidades... y que ademas dos distancias de distintas unidades
	pueden ser iguales... Por lo tanto por qu no hacer esta caracteristica de las distancias explicita?
	Veran que luego se torna muy util
	
	Respecto de como esta escrito el test, para no tener que escribir los assert para oneMeter y oneHundredCentimeters 
	creo un array y lo recorro... Se podria escribir menos aun en este test?
	
	Y por qu se asserta sobre el amount y luego la unidad y no directametne sobre la distancia completa. ej:
	self assert: (aDistance converTo: self meter) = self oneMeter
	"
	
	(Array with: self oneMeter with: self oneHundredCentimeters) do: [ :aDistance |
		self assert: (aDistance convertTo: self meter) amount = 1.
		self assert: (aDistance convertTo: self meter) unit = self meter.
		
		self assert: (aDistance convertTo: self decimeter) amount = 10.
		self assert: (aDistance convertTo: self decimeter) unit = self decimeter.
		
		self assert: (aDistance convertTo: self centimeter) amount = 100.
		self assert: (aDistance convertTo: self centimeter) unit = self centimeter.
		
		self assert: (aDistance convertTo: self millimeter) amount = 1000.
		self assert: (aDistance convertTo: self millimeter) unit = self millimeter.
	
		self assert: (aDistance convertTo: self decameter) amount = (1/10).
		self assert: (aDistance convertTo: self decameter) unit = self decameter.
		
		self assert: (aDistance convertTo: self hectometer) amount = (1/100).
		self assert: (aDistance convertTo: self hectometer) unit = self hectometer.
	
		self assert: (aDistance convertTo: self kilometer) amount = (1/1000).
		self assert: (aDistance convertTo: self kilometer) unit = self kilometer ]! !

!MeasureTest methodsFor: '2 - Converting/accessing/printing' stamp: 'HernanWilkinson 4/25/2011 16:17'!
test07eTimeMeasuresCanBeConvertedToDifferentUnits

	self assert: (self oneMinute convertTo: self second) = self sixtySeconds.
	self assert: (self sixtySeconds convertTo: self minute) = self oneMinute.

	self assert: (self oneHour convertTo: self minute) = self sixtyMinutes.
	self assert: (self sixtyMinutes convertTo: self hour) = self oneHour.

	self assert: (self oneDay convertTo: self hour) = self twentyFourHours.
	self assert: (self twentyFourHours convertTo: self day) = self oneDay.

! !

!MeasureTest methodsFor: '2 - Converting/accessing/printing' stamp: 'HernanWilkinson 4/25/2011 16:17'!
test07fTemperatureMeasuresCanBeConvertedToDifferentUnits

	self assert: (self zeroCelsius convertTo: self fahrenheit) = self thirtyTwoFahrenheits.
	self assert: (self thirtyTwoFahrenheits convertTo: self celsius) = self zeroCelsius.

! !

!MeasureTest methodsFor: '2 - Converting/accessing/printing' stamp: 'HernanWilkinson 4/25/2011 16:19'!
test07gMeasuresCanNotBeConvertedToUnitsOfDifferentDomain

	self should: [ self oneMeter convertTo: self second ] raise: self canNotConvertMeasurementException.
	self should: [ self oneSecond convertTo: self celsius ] raise: self canNotConvertMeasurementException.
	self should: [ self oneCelsius convertTo: self centimeter ] raise: self canNotConvertMeasurementException.


! !

!MeasureTest methodsFor: '3 - Test equality - implementation details' stamp: 'HernanWilkinson 4/25/2011 16:20'!
test08aComparingMeasuresWithOtherObjectsShouldReturnFalse

	"El igual no debe generar ningun tipo de error por mas que se comparen objetos
	de distintos tipos
	Fijense como se crea una instancia de Object sin ningun tipo de problema... qu representa?"

	self deny: self oneMeter = Object new.
	self deny: Object new = self oneMeter! !

!MeasureTest methodsFor: '3 - Test equality - implementation details' stamp: 'HernanWilkinson 4/25/2011 16:21'!
test08bMeasuresThatAreEqualShouldReturnTheSameHash

	"Debido a como funcionan los diccionarios, cuando dos objetos son iguales deben
	tener es mismo hash. La inversa no es verdad"
	
	self assert: self oneMeter hash=self oneMeter hash.
	self assert: self oneMeter hash=self oneHundredCentimeters hash.
	self assert: self oneMinute hash = self sixtySeconds hash.
	self assert: self zeroCelsius hash = self thirtyTwoFahrenheits hash.
	
	
! !

!MeasureTest methodsFor: '3 - Test equality - implementation details' stamp: 'HernanWilkinson 4/25/2011 16:21'!
test08cCopiesOfTheSameMeasureShouldBeEqual

	"Dos objetos que ocupan distintas posiciones de memoria pero representan
	la misma entidad de la realidad, deben devolver true cuando se los compara por igual, puesto
	que representan la misma entidad de la realidad.
	Por lo tanto, el mensaje #= significa 'representan el mismo ente de la realidad?'
	mientras que el mensaje #== significa ocupan la misma posicion de memoria?"

	| originalOneMeter copiedOneMeter |

	"Al objeto oneMeter lo copio para asegurarme de tener uno oneMeter en otra posicion de memoria"

	originalOneMeter := self oneMeter.
	copiedOneMeter := originalOneMeter copy.

	"Me aseguro que ocupen distinta posicion de memoria"
	self deny: originalOneMeter == copiedOneMeter.

	"Ahora me aseguro que sean iguales"
	self assert: originalOneMeter = copiedOneMeter! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'MartinCarreiro 5/26/2013 18:28'!
test09AddingMeasuresOfSameUnitShouldReturnAMeasureRepresentingTheAddingMeasures

	self assert: (self oneMeter + self oneMeter) = self twoMeters.
	"self assert: (self oneSecond + self oneSecond) = self twoSeconds.
	self assert: (self oneCelsius + self oneCelsius) = self twoCelsius."
	

! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:32'!
test10AddingMeasuresOfDifferentUnitShouldReturnAMeasureRepresentingTheAddingMeasures

	self assert: (self oneMeter + self oneHundredCentimeters) = self twoMeters.
	self assert: (self oneMeter + self oneHundredCentimeters) = self twoHundredCentimeters.
	self assert: (self oneSecond + self oneMinute) = self sixtyOneSeconds.

! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:48'!
test10aCanNotAddMeasuresOfDifferentDomain

	self should: [ self oneMeter + self oneSecond ] raise: self canNotConvertMeasurementException 

! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:32'!
test11AddingMeasuresIsCommutative

	self assert: (self oneMeter + self twoMeters)=(self twoMeters + self oneMeter).
	self assert: (self oneSecond + self twoSeconds)=(self twoSeconds + self oneSecond).
	self assert: (self oneCelsius + self twoCelsius)=(self twoCelsius + self oneCelsius)
	


! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:33'!
test12AddingMeasuresIsAssociative

	self assert: 
		((self oneMeter + self oneMeter) + self twoMeters) = 
		(self oneMeter + (self oneMeter + self twoMeters)).

	self assert: 
		((self oneSecond + self oneSecond) + self twoSeconds) = 
		(self oneSecond + (self oneSecond + self twoSeconds)).

	self assert: 
		((self oneCelsius + self oneCelsius) + self twoCelsius) = 
		(self oneCelsius + (self oneCelsius + self twoCelsius)).

		

! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:35'!
test12aAddingMeasuresKeepsMeasureImmutability

	| oneMeter oneSecond oneCelsius |
	
	oneMeter := self oneMeter.
	oneMeter + oneMeter.
	
	self assert: oneMeter = self oneMeter.
	
	oneSecond := self oneSecond.
	oneSecond + oneSecond.
	
	self assert: oneSecond = self oneSecond.
	
	oneCelsius := self oneCelsius.
	oneCelsius + oneCelsius.
	
	self assert: oneCelsius = self oneCelsius 



! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:36'!
test13aNegatingAMeasureNegatesItsAmount

	(Array with: self oneMeter with: self twoSeconds with: self thirtyTwoFahrenheits) do: [:aMeasure |
		self assert: aMeasure negated amount=aMeasure amount negated ]
! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:36'!
test13bNegatingAMeasureIsReflective

	(Array with: self oneMeter with: self twoSeconds with: self thirtyTwoFahrenheits) do: [:aMeasure |
		self assert: aMeasure negated negated=aMeasure ].
! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:37'!
test14SubtractingMeasuresOfSameUnitShouldReturnAMeasureRepresentingTheSubtractedMeasures

	self assert: (self twoMeters - self oneMeter) = self oneMeter.
	self assert: (self oneMeter - self oneMeter) = self zeroMeters.
	self assert: (self oneMeter - self twoMeters) = self oneMeter negated.
	
	self assert: (self twoSeconds - self oneSecond) = self oneSecond.
	self assert: (self oneSecond - self oneSecond) = self zeroSeconds.
	self assert: (self oneSecond - self twoSeconds) = self oneSecond negated.
	
	self assert: (self twoCelsius - self oneCelsius) = self oneCelsius.
	self assert: (self oneCelsius - self oneCelsius) = self zeroCelsius.
	self assert: (self oneCelsius - self twoCelsius) = self oneCelsius negated.
! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:38'!
test15SubtractingMeasuresOfDifferentUnitShouldReturnAMeasureRepresentingTheSubtractedMeasures

	self assert: (self twoMeters - self oneHundredCentimeters) = self oneMeter.
	self assert: (self oneMeter - self oneHundredCentimeters) = self zeroMeters.
	self assert: (self oneHundredCentimeters - self twoMeters) = self oneMeter negated.
	
	self assert: (self sixtyOneSeconds - self oneMinute) = self oneSecond.
	

	! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:48'!
test15aCanNotSubtractMeasuresOfDifferentDomain

	self should: [ self oneMeter - self oneSecond ] raise: self canNotConvertMeasurementException 

! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:38'!
test16SubtractingMeasuresIsAssociative

	self assert: 
		((self twoMeters - self oneMeter) - self oneMeter) = 
		(self twoMeters - (self oneMeter + self oneMeter)).

	self assert: 
		((self twoSeconds - self oneSecond) - self oneSecond) = 
		(self twoSeconds - (self oneSecond + self oneSecond)).	

	

	! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:39'!
test16aSubtractingMeasuresKeepsMeasuresImmutability

	| oneMeter oneSecond oneCelsius |
	
	oneMeter := self oneMeter.
	oneMeter - oneMeter.
	
	self assert: oneMeter = self oneMeter.
	
	oneSecond := self oneSecond.
	oneSecond - oneSecond.
	
	self assert: oneSecond = self oneSecond.
	
	oneCelsius := self oneCelsius.
	oneCelsius - oneCelsius.
	
	self assert: oneCelsius = self oneCelsius 


! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:40'!
test17aMeasuresCanBeMultiplyByAnInteger

	self assert: self oneMeter * 1 = self oneMeter.
	self assert: self oneMeter * 2 = self twoMeters.
	self assert: self oneMeter * 0 = self zeroMeters.

	self assert: self oneSecond * 1 = self oneSecond.
	self assert: self oneSecond * 2 = self twoSeconds.
	self assert: self oneSecond * 0 = self zeroSeconds.

	self assert: self oneCelsius * 1 = self oneCelsius.
	self assert: self oneCelsius * 2 = self twoCelsius.
	self assert: self oneCelsius * 0 = self zeroCelsius.
! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:40'!
test17bMeasuresCanBeMultiplyByAFraction

	self assert: (self oneMeter * (2/3)) amount = (2/3).
	self assert: (self twoMeters * (2/3)) amount = (2 * (2/3)).

	self assert: (self oneSecond * (2/3)) amount = (2/3).
	self assert: (self twoSeconds * (2/3)) amount = (2 * (2/3)).

	self assert: (self oneCelsius * (2/3)) amount = (2/3).
	self assert: (self twoCelsius * (2/3)) amount = (2 * (2/3)).
! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:41'!
test17cMeasuresCanBeMultiplyByAFloat

	self assert: (self oneMeter * 2.5) amount = 2.5.
	self assert: (self twoMeters * 2.5) amount = (2 * 2.5).

	self assert: (self oneSecond * 2.5) amount = 2.5.
	self assert: (self twoSeconds * 2.5) amount = (2 * 2.5).

	self assert: (self oneCelsius * 2.5) amount = 2.5.
	self assert: (self twoCelsius * 2.5) amount = (2 * 2.5).
! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:45'!
test17dMeasuresCanBeMultiplyByAMeasureOfSameDomain

	self assert: (self oneMeter * self oneMeter) = self oneSquaredMeter.
	self assert: (self twoMeters * self twoMeters) = self fourSquaredMeters.
	self assert: (self twoMeters * self twoMeters * self twoMeters) = self eightCubicMeters.

	self assert: (self oneSecond * self oneSecond) = self oneSquaredSecond.
! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:53'!
test17eMeasuresCanBeMultiplyByAMeasureOfDifferentDomain

	self assert: (self oneMeter * self twoSeconds) = self twoMetersTimesSecond.
! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:42'!
test18MultiplicationKeepsMeasureImmutability

	| oneMeter oneSecond oneCelsius |
	
	oneMeter := self oneMeter.
	oneMeter * 2.
	
	self assert: oneMeter = self oneMeter.
	
	oneSecond := self oneSecond.
	oneSecond * 2.
	
	self assert: oneSecond = self oneSecond.
	
	oneCelsius := self oneCelsius.
	oneCelsius * 2.
	
	self assert: oneCelsius = self oneCelsius 




! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:49'!
test19aMeasuresCanBeDividedByAnInteger

	self assert: (self oneMeter / 1) = self oneMeter.
	self assert: (self twoMeters / 2) = self oneMeter.

	self assert: (self oneSecond / 1) = self oneSecond.
	self assert: (self twoSeconds / 2) = self oneSecond.

	self assert: (self oneCelsius / 1) = self oneCelsius.
	self assert: (self twoCelsius / 2) = self oneCelsius.

! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:50'!
test19bMeasuresCanBeDividedByAFraction

	self assert: (self twoMeters / (1/3)) amount = 6.
	self assert: (self twoSeconds / (1/3)) amount = 6.
	self assert: (self twoCelsius / (1/3)) amount = 6.




! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:50'!
test19cMeasuresCanBeDividedByAFloat

	self assert: (self twoMeters / 2.5) amount = (2/2.5).
	self assert: (self twoSeconds / 2.5) amount = (2/2.5).
	self assert: (self twoCelsius / 2.5) amount = (2/2.5).




! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:52'!
test19dMeasuresCanBeDividedByAMeasureOfSameDomain

	self assert: (self oneMeter / self oneMeter) = 1.
	self assert: (self twoMeters / self oneMeter) = 2.
		
	self assert: (self oneSecond / self oneSecond) = 1.
! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:52'!
test19eMeasuresCanBeDividedByAMeasureOfDifferentDomain

	self assert: (self oneMeter / self twoSeconds) = self twoMetersBySecond.
! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:54'!
test20DivisionKeepsMeasureImmutability

	| oneMeter oneSecond oneCelsius |
	
	oneMeter := self oneMeter.
	oneMeter / 2.
	
	self assert: oneMeter = self oneMeter.
	
	oneSecond := self oneSecond.
	oneSecond / 2.
	
	self assert: oneSecond = self oneSecond.
	
	oneCelsius := self oneCelsius.
	oneCelsius / 2.
	
	self assert: oneCelsius = self oneCelsius 


! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'efo 5/2/2011 20:56'!
test20aFinalSpeedCanBeRepresentedWithMeasures

	self assert: 
		(self finalSpeedStartingAt: self oneMeterPerSecond accelerating: self twoMetersPerSquaredSecond during: self sixtySeconds) = 
		(self oneMeterPerSecond + (self twoMetersPerSquaredSecond * self sixtySeconds)).
		
	self assert: (self finalSpeedStartingAt: self oneMeterPerSecond accelerating: self twoMetersPerSquaredSecond during: self sixtySeconds) = self oneHundredTwentyOneMetersPerSecond 
	
! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 17:00'!
test20bMultiplyingANumberByAUnitReturnsAMeasure

	self assert: (1*self meter) = self oneMeter.
	self assert: (2*self second) = self twoSeconds.
! !

!MeasureTest methodsFor: '5 - Test magnitude protocol' stamp: 'HernanWilkinson 4/25/2011 17:02'!
test21MeasuresProvideATotalOrder

	self assert: (self measureExamples allSatisfy: [ :aMeasure | aMeasure < (aMeasure + (1* aMeasure unit))])

! !

!MeasureTest methodsFor: '5 - Test magnitude protocol' stamp: 'HernanWilkinson 4/25/2011 17:03'!
test22MeasuresCanBeComparedWithGreaterThan

	self assert: (self measureExamples allSatisfy: [ :aMeasure | (aMeasure + (1 * aMeasure unit)) > aMeasure ])
! !

!MeasureTest methodsFor: '5 - Test magnitude protocol' stamp: 'HernanWilkinson 4/25/2011 17:04'!
test23MeasuresCanBeComparedWithLessThanOrEqual

	self assert: (self measureExamples allSatisfy: [ :aMeasure | aMeasure <= aMeasure and: [aMeasure <= (aMeasure + (1 * aMeasure unit))]])

	
! !

!MeasureTest methodsFor: '5 - Test magnitude protocol' stamp: 'HernanWilkinson 4/25/2011 17:05'!
test24MeasuresCanBeComparedWithGreaterThanOrEqual

	self assert: (self measureExamples allSatisfy: [ :aMeasure | aMeasure >= aMeasure and: [(aMeasure + (1 * aMeasure unit)) >= aMeasure]])! !

!MeasureTest methodsFor: '5 - Test magnitude protocol' stamp: 'HernanWilkinson 4/25/2011 17:05'!
test25MaxReturnsTheMaximumBetweenTwoMeasures

	self assert: (self twoMeters max: self oneMeter) = self twoMeters.
	self assert: (self oneMeter max: self twoMeters) = self twoMeters.

	self assert: (self twoMeters max: self oneHundredCentimeters) = self twoMeters.
	self assert: (self oneHundredCentimeters max: self twoMeters) = self twoMeters.

	
! !

!MeasureTest methodsFor: '5 - Test magnitude protocol' stamp: 'HernanWilkinson 4/25/2011 17:05'!
test26MinReturnsTheMinimumBetweenTwoMeasures

	self assert: (self twoMeters min: self oneMeter) = self oneMeter.
	self assert: (self oneMeter min: self twoMeters) = self oneMeter.

	self assert: (self twoMeters min: self oneHundredCentimeters) = self oneHundredCentimeters.
	self assert: (self oneHundredCentimeters min: self twoMeters) = self oneHundredCentimeters.

	
! !

!MeasureTest methodsFor: '5 - Test magnitude protocol' stamp: 'HernanWilkinson 4/25/2011 17:05'!
test27BetweenAndReturnsTrueIfAMeasureIsBetweenOtherTwoMeasures

	self assert: (self oneMeter between: self zeroMeters and: self twoMeters).
	self assert: (self oneHundredCentimeters between: self zeroMeters and: self twoMeters).
	
	self deny: (self twoMeters between: self zeroMeters and: self oneMeter).
	self deny: (self twoHundredCentimeters between: self zeroMeters and: self oneMeter).

! !

!MeasureTest methodsFor: '6 - Test interval protocol' stamp: 'HernanWilkinson 4/25/2011 17:09'!
test28IntervalOfMeasuresCanBeCreatedBecauseTheyDefineATotalOrder

	self assert: (self oneMeter to: self twoMeters) first=self oneMeter.
	self assert: (self oneMeter to: self twoMeters) last=self twoMeters.
	self assert: (self oneMeter to: self twoMeters) increment =self oneMeter.	

	self assert: (self oneHundredCentimeters to: self twoMeters) first=self oneMeter.
	self assert: (self oneHundredCentimeters to: self twoMeters) last=self twoMeters.
	self assert: (self oneHundredCentimeters to: self twoMeters) increment=self oneCentimeter.	
	! !

!MeasureTest methodsFor: '6 - Test interval protocol' stamp: 'HernanWilkinson 4/25/2011 17:16'!
test29MeasureIntervalStepCanBeAnyMeasure

	self assert: (self zeroMeters to: (self twoMeters * 2) by: self twoMeters) first=self zeroMeters.
	self assert: (self zeroMeters to: (self twoMeters * 2) by: self twoMeters) last=(self twoMeters * 2).
	self assert: (self zeroMeters to: (self twoMeters * 2) by: self twoMeters) increment=self twoMeters.

	self assert: ((self twoMeters * 2) to: self zeroMeters by: self twoMeters negated) first=(self twoMeters * 2).
	self assert: ((self twoMeters * 2) to: self zeroMeters by: self twoMeters negated) last=self zeroMeters.
	self assert: ((self twoMeters * 2) to: self zeroMeters by: self twoMeters negated) increment=self twoMeters negated

	! !

!MeasureTest methodsFor: '6 - Test interval protocol' stamp: 'HernanWilkinson 4/25/2011 17:17'!
test30MeasureIntervalSizeReturnsTheNumberOfMeasuresOfTheInterval

	self assert: (self zeroMeters to: (self twoMeters * 2) ) size =5.
	self assert: ((self twoMeters * 2) to: self zeroMeters by: self oneMeter negated) size =5.
! !

!MeasureTest methodsFor: '6 - Test interval protocol' stamp: 'HernanWilkinson 4/25/2011 17:17'!
test31MeasureIntervalCanBeEnumeratedUsingDo

	| value |
	
	value := 1.
	(self oneMeter to: (self oneMeter * 10)) do: [ :aMeasure |
		self assert: aMeasure amount = value.
		value := value + 1 ].
	
	self assert: value = 11
! !

!MeasureTest methodsFor: '6 - Test interval protocol' stamp: 'HernanWilkinson 4/25/2011 17:18'!
test32MeasureIntervalCanBeEnumeratedWithSpecificStep

	| value initialDistance byDistance finalDistance |
	
	initialDistance := self oneMeter.
	byDistance := self twoMeters.
	finalDistance := initialDistance * 9.
	value := initialDistance amount.
		
	(initialDistance to: finalDistance by:  byDistance) do: [ :aMeasure |
		self assert: aMeasure amount = value.
		value := value + byDistance amount ].
	
	self assert:  value = (finalDistance amount + byDistance amount)
! !

!MeasureTest methodsFor: '7 - Polymorphic zero' stamp: 'HernanWilkinson 4/25/2011 17:20'!
test33NothingOnAnyDomainAreEquals

	self assert: self zeroMeters = self zeroSeconds.
	self assert: self zeroSeconds = self zeroKelvin.
	self assert: self zeroKelvin = self zeroMeters! !

!MeasureTest methodsFor: '7 - Polymorphic zero' stamp: 'HernanWilkinson 4/25/2011 17:42'!
test34ZeroCelsiusIsNotNothingOnTemperature

	self deny: self zeroCelsius = self zeroKelvin.
	self deny: self zeroSeconds = self zeroCelsius.
! !

!MeasureTest methodsFor: 'temperature measures' stamp: 'PabloRago 5/20/2013 20:44'!
thirtyTwoFahrenheits
	
	" 'Este metodo debe devolver un objeto que represente 32 fahrenhetis, o sea
	0 grados centigrados'"
	
	^ Measure newMeasure: 32 withUnit: self fahrenheit ! !

!MeasureTest methodsFor: 'time measures' stamp: 'HernanWilkinson 4/25/2011 17:29'!
timeExamples

	^self objectsInCategoryOf: thisContext method selector.
	
! !

!MeasureTest methodsFor: 'time units' stamp: 'HernanWilkinson 4/29/2011 17:10'!
timeUnits

	^self objectsInCategoryOf: thisContext method selector.
	
 ! !

!MeasureTest methodsFor: 'time measures' stamp: 'PabloRago 5/20/2013 20:48'!
twentyFourHours
	
	^ Measure newMeasure: 24 withUnit: self hour! !

!MeasureTest methodsFor: 'temperature measures' stamp: 'PabloRago 5/20/2013 20:45'!
twoCelsius
	
	^ Measure newMeasure: 2 withUnit: self celsius ! !

!MeasureTest methodsFor: 'distance measures' stamp: 'PabloRago 5/20/2013 20:33'!
twoHundredCentimeters
	
	^ Measure newMeasure: 200 withUnit: self centimeter ! !

!MeasureTest methodsFor: 'distance measures' stamp: 'PabloRago 5/20/2013 20:33'!
twoMeters
	
	^ Measure newMeasure: 2 withUnit: self meter ! !

!MeasureTest methodsFor: 'compound measures' stamp: 'HernanWilkinson 4/25/2011 17:35'!
twoMetersBySecond

	^self error: 'este metodo debe devolver la medida 2 metros/segundo'! !

!MeasureTest methodsFor: 'compound measures' stamp: 'HernanWilkinson 4/25/2011 17:33'!
twoMetersPerSquaredSecond

	^self error: 'este metodo debe devolver la medida 2 metros/segundoAlCuadrado'! !

!MeasureTest methodsFor: 'compound measures' stamp: 'HernanWilkinson 4/25/2011 17:33'!
twoMetersTimesSecond

	^self error: 'este metodo debe devolver la medida 2 metros*segundo'! !

!MeasureTest methodsFor: 'time measures' stamp: 'PabloRago 5/20/2013 20:48'!
twoSeconds
	
	^ Measure newMeasure: 2 withUnit: self second ! !

!MeasureTest methodsFor: 'objects in category' stamp: 'HernanWilkinson 4/25/2011 17:34'!
units 

	^ OrderedCollection new
		addAll: self distanceUnits;
		addAll: self temperatureUnits;
		addAll: self timeUnits;
		yourself ! !

!MeasureTest methodsFor: 'temperature measures' stamp: 'PabloRago 5/20/2013 20:45'!
zeroCelsius
	
	^ Measure newMeasure: 0 withUnit: self celsius  ! !

!MeasureTest methodsFor: 'temperature measures' stamp: 'PabloRago 5/20/2013 20:45'!
zeroKelvin
	
	^ Measure newMeasure: 0 withUnit: self kelvin ! !

!MeasureTest methodsFor: 'distance measures' stamp: 'PabloRago 5/20/2013 20:34'!
zeroMeters

	^ Measure newMeasure: 0 withUnit: self meter ! !

!MeasureTest methodsFor: 'time measures' stamp: 'PabloRago 5/20/2013 20:48'!
zeroSeconds
	
	^ Measure newMeasure: 0 withUnit: self second ! !

Error subclass: #CanNotConvertMeasurementException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-Measure'!

Object subclass: #Unit
	instanceVariableNames: 'name domain normal'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-Measure'!

Unit subclass: #NonProportionalUnit
	instanceVariableNames: 'conversionBlock inverseConversionBlock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-Measure'!

!NonProportionalUnit methodsFor: 'as yet unclassified' stamp: 'MartinCarreiro 5/24/2013 13:35'!
convertAmount: anAmount to: aUnit 
	
	^aUnit inverseConversionBlock value: (self normalize: anAmount).
	! !

!NonProportionalUnit methodsFor: 'as yet unclassified' stamp: 'MartinCarreiro 5/26/2013 17:40'!
initializeUnit: aName withConverter: aConvertionBlock andDomain: aDomain andInverseConverter: anotherConversionBlock andNormal: aNormal

	name := aName.
	conversionBlock := aConvertionBlock.
	inverseConversionBlock := anotherConversionBlock.
	domain := aDomain.
	normal := aNormal.! !

!NonProportionalUnit methodsFor: 'as yet unclassified' stamp: 'MartinCarreiro 5/24/2013 13:11'!
inverseConversionBlock

	^inverseConversionBlock ! !

!NonProportionalUnit methodsFor: 'as yet unclassified' stamp: 'PabloRago 5/20/2013 21:21'!
normalize: anAmount 
	^ conversionBlock value: anAmount! !

Unit subclass: #ProportionalUnit
	instanceVariableNames: 'factor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-Measure'!

!ProportionalUnit methodsFor: 'as yet unclassified' stamp: 'PabloRago 5/26/2013 15:28'!
convertAmount: anAmount to: aUnit 
	
	^((self normalize: anAmount) / aUnit factor) asFraction.! !

!ProportionalUnit methodsFor: 'as yet unclassified' stamp: 'MartinCarreiro 5/24/2013 09:29'!
factor
	^factor! !

!ProportionalUnit methodsFor: 'initialize-release' stamp: 'MartinCarreiro 5/26/2013 17:40'!
initializeUnit: aName withFactor: aFactor andDomain: aDomain  andNormal: aNormal

	name := aName.
	factor := aFactor.
	domain := aDomain.
	normal := aNormal.! !

!ProportionalUnit methodsFor: 'as yet unclassified' stamp: 'PabloRago 5/26/2013 15:35'!
normalize: anAmount
	^(anAmount * factor) asFraction.! !

!Unit class methodsFor: 'Temperature units' stamp: 'MartinCarreiro 5/26/2013 20:52'!
celsius

	"De Celsius a Kelvin y de kelvin a celcius"
	^Unit newUnit: 'celsius' withConverter: [ :anAmount | anAmount+273. ] andDomain: 'temperature' andInverseConverter: [ :anAmount | anAmount - 273 ] andNormal: 'kelvin' asSymbol  ! !

!Unit class methodsFor: 'Distance units' stamp: 'MartinCarreiro 5/26/2013 20:51'!
centimeter

	^ Unit newUnit: 'centimeter' withFactor: 0.01 andDomain: 'distance' andNormal: 'meter' asSymbol 
	
! !

!Unit class methodsFor: 'Time units' stamp: 'MartinCarreiro 5/26/2013 20:52'!
day

	^ Unit newUnit: 'day' withFactor: 86400 andDomain: 'time' andNormal: 'second' asSymbol
! !

!Unit class methodsFor: 'Distance units' stamp: 'MartinCarreiro 5/26/2013 20:51'!
decameter

	^ Unit newUnit: 'decameter' withFactor: 10 andDomain: 'distance' andNormal:'meter' asSymbol  
! !

!Unit class methodsFor: 'Distance units' stamp: 'MartinCarreiro 5/26/2013 20:51'!
decimeter

	^ Unit newUnit: 'decimeter' withFactor: 0.1 andDomain: 'distance' andNormal: 'meter' asSymbol 
! !

!Unit class methodsFor: 'Temperature units' stamp: 'MartinCarreiro 5/26/2013 20:52'!
fahrenheit

	"De fahrenheit a Kelvin y de kelvin a fahrenheit"
	^Unit newUnit: 'fahrenheit' withConverter: [ :anAmount | ((anAmount-32)*5/9)+273. ] andDomain: 'temperature' andInverseConverter: [ :anAmount | (9/5)*(anAmount - 273) + 32 ] andNormal: 'kelvin' asSymbol! !

!Unit class methodsFor: 'Distance units' stamp: 'MartinCarreiro 5/26/2013 20:51'!
hectometer

	^ Unit newUnit: 'hectometer' withFactor: 100 andDomain: 'distance' andNormal: 'meter' asSymbol  
! !

!Unit class methodsFor: 'Time units' stamp: 'MartinCarreiro 5/26/2013 20:52'!
hour

	^ Unit newUnit: 'hour' withFactor: 3600 andDomain: 'time' andNormal: 'second' asSymbol
! !

!Unit class methodsFor: 'Temperature units' stamp: 'MartinCarreiro 5/26/2013 20:52'!
kelvin

	"De Kelvin a Kelvin y kelvin a kelvin"
	^Unit newUnit: 'kelvin' withConverter: [ :anAmount | anAmount. ] andDomain: 'temperature' andInverseConverter: [ :anAmount | anAmount ] andNormal: 'kelvin' asSymbol! !

!Unit class methodsFor: 'Distance units' stamp: 'MartinCarreiro 5/26/2013 20:51'!
kilometer

	^ Unit newUnit: 'kilometer' withFactor: 1000 andDomain: 'distance' andNormal: 'meter' asSymbol  
! !

!Unit class methodsFor: 'Distance units' stamp: 'MartinCarreiro 5/26/2013 20:48'!
meter

	^ Unit newUnit: 'meter' withFactor: 1 andDomain: 'distance' andNormal: 'meter' asSymbol 
! !

!Unit class methodsFor: 'Distance units' stamp: 'MartinCarreiro 5/26/2013 20:51'!
millimeter

	^ Unit newUnit: 'milimeter' withFactor: 0.001 andDomain: 'distance' andNormal: 'meter' asSymbol  
! !

!Unit class methodsFor: 'Time units' stamp: 'MartinCarreiro 5/26/2013 20:52'!
minute

	^ Unit newUnit: 'minute' withFactor: 60 andDomain: 'time' andNormal: 'second' asSymbol
! !

!Unit class methodsFor: 'as yet unclassified' stamp: 'MartinCarreiro 5/26/2013 17:40'!
newUnit: aName withConverter: aConversionBlock andDomain: aDomain andInverseConverter: anotherConversionBlock andNormal: aNormal

	|aUnit|

	 aUnit := NonProportionalUnit new.
	^aUnit initializeUnit: aName withConverter: aConversionBlock andDomain: aDomain andInverseConverter: anotherConversionBlock andNormal: aNormal. ! !

!Unit class methodsFor: 'as yet unclassified' stamp: 'MartinCarreiro 5/26/2013 17:39'!
newUnit: aName withFactor: aFactor andDomain: aDomain andNormal: aNormal

	|aUnit|

	 aUnit := ProportionalUnit new.
	^aUnit initializeUnit: aName withFactor: aFactor andDomain: aDomain andNormal: aNormal. ! !

!Unit class methodsFor: 'Time units' stamp: 'MartinCarreiro 5/26/2013 20:52'!
second

	^ Unit newUnit: 'second' withFactor: 1 andDomain: 'time' andNormal: 'second' asSymbol
! !

!Unit methodsFor: 'comparing' stamp: 'PabloRago 5/12/2013 13:18'!
= anotherUnit
	^ name = anotherUnit name.! !

!Unit methodsFor: 'converting' stamp: 'PabloRago 5/26/2013 18:32'!
adaptToNumber: aNumber andSend: aMessage
	
	^ (self newUnitaryMeasure) perform: aMessage with: aNumber.  ! !

!Unit methodsFor: 'getters' stamp: 'MartinCarreiro 5/24/2013 13:20'!
convertAmount: anAmount to: aUnit 
	
	^self subclassResponsibility .! !

!Unit methodsFor: 'initialize-release' stamp: 'MartinCarreiro 5/13/2013 19:17'!
copy

	^ Unit newUnit: self name withFactor: self factor .! !

!Unit methodsFor: 'getters' stamp: 'PabloRago 5/20/2013 20:24'!
domain
	^ domain.! !

!Unit methodsFor: 'getters' stamp: 'PabloRago 5/12/2013 12:58'!
name 
	^name.! !

!Unit methodsFor: 'as yet unclassified' stamp: 'PabloRago 5/26/2013 18:31'!
newUnitaryMeasure

	^ Measure newMeasure: 1 withUnit: self.! !

!Unit methodsFor: 'getters' stamp: 'MartinCarreiro 5/26/2013 20:50'!
normal 
"	^normal."
	^ Unit perform: normal .! !

!Unit methodsFor: 'getters' stamp: 'PabloRago 5/12/2013 12:50'!
printString
	^ name.! !
